// Copyright (C) 2009 - Digiteo - Michael Baudin
// Copyright (C) INRIA - Farid BELAHCENE
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

//
// number_primes --
//   Returns the list of all primes from 1 up to n,
//   as a row matrix.
//
function result = number_primes ( n , method )
  if (~isdef('method','local')) then
    method = "erathostenefast";
  end
  if type(n)<>1 then
    error(msprintf(gettext("%s: Wrong type for input argument #%d: A real expected.\n"),"primes"));
  end	
  if ( prod(size(n,"*"))<>1 ) then
    error(msprintf(gettext("%s: Wrong size for input argument #%d: A real expected.\n"),"primes"));
  end	
  if ( n < 1 )
    result = [];
    return
  end
  select method
  case "erathostenefast"
    result = erathostenefast ( n )
  case "erathostenenaive"
    result = erathostenenaive ( n )
  else
    errmsg = sprintf ( gettext ( "%s: Unknown method %s" ) , "number_primes" , method )
    error(errmsg)
  end
endfunction
//
// erathostenefast --
//   Returns the list of all primes from 1 up to n.
//   Uses a fast algorithm, based on vectorized Scilab.
//
function y = erathostenefast ( x )
	y = (1:fix(x));
	ylength = size(y,'*')
	i = 2
	while (i <= sqrt(x))
		if y(i)<> 0
			y(2*y(i):y(i):ylength) = 0
  		end
  		i = i+1
	end
 	y(y<=1)=[];
endfunction
//
// erathostenenaive --
//   Returns the list of all primes from 1 up to n.
//   Uses a straitforward algorithm.
//
function result = erathostenenaive ( n )
    sieve(1) = %f
    for i=2:n
      sieve(i) = %t
    end
    p = 2
    while ( p * p <= n )
      i = p * p
      while ( i<=n )
        sieve(i) = %f
        i = i + p
      end
      p = p + 1 
    end
    result = []
    for i = 1:n
      if ( sieve(i) ) then
        result(1,$+1) = i
      end
    end
endfunction

